unit sPanel;
(************************************************************************
sStyle components package
Copyright (c) 2002 Serge Goncharov
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.1, 04-02-2002
*************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, sStyleUtil, sConst, sGraphUtils, sUtils, sCheckedControl, Dsgnintf,
  imglist, comctrls;

type

  TsPanel = class;

  TsTransparentPanel = class(TCustomPanel) // Abstract class, sStyle dont created
  private
  protected
    FForbidMouse : boolean;
    procedure WMEraseBkGND (var Message: TWMPaint); message WM_ERASEBKGND;
    procedure PaintPanelBorder(DC : longint; Left, Top, Right, Bottom: integer; InvertColors : boolean; sStyle : TsPaintStyle);
    procedure WriteText(R : TRect; sStyle: TsPaintStyle); dynamic;
    function Margin: integer;
    procedure Invalidate; override;
  public
    constructor Create (AOwner: TComponent); override;
    property BorderStyle;
  published
    property Align;
    property Alignment;
    property Anchors;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property Caption;
    property Constraints;
    property UseDockManager default True;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property DockManager;
    property Enabled;
    property FullRepaint;
    property Font;
    property ForbidMouse: boolean read FForbidMouse write FForbidMouse default True;
    property Locked;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
  end;

  TsCustomPanel = class(TsTransparentPanel)
  private
  protected
    FWallpaperCheck: boolean;
    FsStyle : TsPassiveBGStyle;
    procedure DrawPanelBorders(Canvas: TCanvas);
    procedure WndProc (var Message: TMessage); override;
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
  published
    property BorderStyle;
    property sStyle : TsPassiveBGStyle read FsStyle write FsStyle;
    property WallpaperCheck: boolean read FWallpaperCheck write FWallpaperCheck default False;
  end;

  TsPanel = class(TsCustomPanel)
  private
  protected
  public
    constructor Create (AOwner: TComponent); override;
  published
  end;

  TsContainer = class(TsCustomPanel)
  private
  protected
  public
    constructor Create (AOwner: TComponent); override;
  published
    property BorderStyle default bsSingle;
  end;

  TsToolBar = class(TsCustomPanel)
  private
    FAutoSize : boolean;
    FImages : TCustomImageList;
    FImagesHot : TCustomImageList;
    FImagesDisabled : TCustomImageList;
    FShowCaptions : boolean;
    FButtonWidth: integer;
    FButtonHeight: integer;
    procedure SetImages(Index: integer; Value: TCustomImageList);
    procedure SetShowCaptions(const Value: boolean);
    procedure SetAutoSize(const Value: boolean);
  protected
  public
    DontAutoSize : boolean;
    procedure AdjustSize; override;
    constructor Create (AOwner: TComponent); override;
  published
    property AutoSize : boolean read FAutoSize write SetAutoSize;
    property ButtonWidth : integer read FButtonWidth write FButtonWidth default 22;
    property ButtonHeight : integer read FButtonHeight write FButtonHeight default 22;
    property ImagesDisabled : TCustomImageList index 2 read FImagesDisabled write SetImages;
    property ImagesHot : TCustomImageList index 1 read FImagesHot write SetImages;
    property Images : TCustomImageList index 0 read FImages write SetImages;
    property ShowCaptions : boolean read FShowCaptions write SetShowCaptions;
  end;

  TsGlassPanel = class(TsCustomPanel)
  private
    FMask : TPicture;
    procedure SetMask(const Value: TPicture);
    procedure DrawGlassBorders;
  protected
  public
    constructor Create (AOwner: TComponent); override;
    procedure Paint; override;
  published
    property Mask: TPicture read FMask write SetMask;
  end;

  TsToolBarEditor = class(TDefaultEditor)
  protected
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

const
  Alignments: array[TAlignment] of Longint = (DT_LEFT, DT_RIGHT, DT_CENTER);

implementation

uses sPageControl, sCustomButton, sVclUtils, sEdit, sCurrencyEdit;

{ TsTransparentPanel }

constructor TsTransparentPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
//  FsStyle.COC := COC_TsTransparentPanel;
  ControlStyle := ControlStyle + [csOpaque];
  FForbidMouse := True;
end;

procedure TsTransparentPanel.Invalidate;
begin
  if Showing then begin
    inherited;
  end;
end;

function TsTransparentPanel.Margin: integer;
begin
  Result := BorderWidth;
  if BevelOuter <> bvNone then inc(Result, BevelWidth);
  if BevelInner <> bvNone then inc(Result, BevelWidth);
end;

procedure TsTransparentPanel.PaintPanelBorder(DC, Left, Top, Right,
  Bottom: integer; InvertColors: boolean; sStyle: TsPaintStyle);
var
  GetColor : array [0..1] of TColor;
  R: TRect;
  Width, i : integer;
  Color1, Color2 : TColor;
  procedure DrawRect; begin
    if Enabled then
      DrawRectangleOnDC(DC, R, ColorToRGB(Color1), Color2, Width)
    else begin
      DrawRectangleOnDC(DC, R, ColorToRGB(Color2), Color2, Width);
    end;
  end;
begin
  GetColor[1] := ColorToRGB(sStyle.Painting.ColorBorderTop);
  GetColor[0] := ColorToRGB(sStyle.Painting.ColorBorderBottom);
  R := Rect(Left, Top, Right, Bottom);

  Color1 := GetColor[integer(not InvertColors)];
  Color2 := GetColor[integer(InvertColors)];
  
  if sStyle.ControlIsActive and (msBorder in sStyle.Selection.Mode) then begin
    Case sStyle.Selection.Border.Bevel of
      bsFlat : begin
        width := sStyle.Selection.Border.Width;
        DrawRect;
      end;
      sConst.bsRaised: begin
        for i := 1 to sStyle.Selection.Border.Width do begin
          width := 1;
          Color1 := LightingColor(GetColor[integer(not InvertColors)], 180, i / sStyle.Selection.Border.Width);
          Color2 := LightingColor(GetColor[integer(InvertColors)], 20, i / sStyle.Selection.Border.Width);
          DrawRect;
        end;
      end;
      sConst.bsLowered: begin
        for i := 1 to sStyle.Selection.Border.Width do begin
          width := 1;
          Color1 := DarkingColor(GetColor[integer(not InvertColors)], 0, i / sStyle.Selection.Border.Width);
          Color2 := DarkingColor(GetColor[integer(InvertColors)], 255, i / sStyle.Selection.Border.Width);
          DrawRect;
        end;
      end;
    end;
  end
  else begin
    width := BevelWidth;
    Color1 := GetColor[integer(not InvertColors)];
    Color2 := GetColor[integer(InvertColors)];
    DrawRect;
  end;
end;

procedure TsTransparentPanel.WMEraseBkGND(var Message: TWMPaint);
begin
  Message.Result := 1;
end;

procedure TsTransparentPanel.WriteText(R: TRect; sStyle: TsPaintStyle);
var
  Flags: Longint;
begin
  sStyle.FCacheBmp.Canvas.Brush.Style := bsClear;
  sStyle.FCacheBmp.Canvas.Font := Font;

  R.Top := ((R.Bottom + R.Top) - sStyle.FCacheBmp.Canvas.TextHeight('W')) div 2;
  R.Bottom := R.Top + sStyle.FCacheBmp.Canvas.TextHeight('W');

  Flags := DrawTextBiDiModeFlags(DT_EXPANDTABS or DT_VCENTER or Alignments[Alignment]);

  sStyle.WriteText(sStyle.FCacheBmp.Canvas, PChar(Caption), Enabled,
                            R.Left, R.Top, R.Right - R.Left, R.Bottom - R.Top, Flags);
end;

{ TsCustomPanel }

constructor TsCustomPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  sStyle := TsPassiveBGStyle.Create(Self);
  sStyle.COC := COC_TsCustomPanel;
  FWallpaperCheck := False;
end;

destructor TsCustomPanel.Destroy;
begin
  FreeAndNil(FsStyle);
  inherited Destroy;
end;

procedure TsCustomPanel.Paint;
var
  aRect: TRect;
  i: Integer;
  RStep, GStep, BStep: real;
  R, G, B: real;
begin
  if (csDestroying in ComponentState) then exit;
//  if not sStyle.FDrawingStop then begin
    aRect := ClientRect;
    inc(aRect.Right);
    inc(aRect.Bottom);
    sStyle.FCacheBmp.Height := aRect.Bottom - aRect.Top;
    sStyle.FCacheBmp.Width := aRect.Right - aRect.Left;

    if (GetStyleInfo(Parent) < tos_SPAINTSTYLE) and (sStyle.Background.Mode = bgsTransparent) then begin
      sStyle.Background.Mode := bgsNone;
    end;

    case sStyle.Background.Mode of
      bgsNone: begin
        Color := sStyle.GetActiveColor;
        sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
        sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
        sStyle.FCacheBmp.Canvas.Brush.Color := sStyle.GetActiveColor;
        sStyle.FCacheBmp.Canvas.Rectangle(aRect);
      end;
      bgsTransparent: begin
        if sStyle.Painting.Transparency = 0 then begin
          sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
          sStyle.FCacheBmp.Canvas.Brush.Color := Color;
          sStyle.FCacheBmp.Canvas.FillRect(aRect);
        end
        else begin
          if sStyle.GetParentCache <> nil then begin
            if sStyle.Shadow.Enabled then begin
              // Copying area
              FadeRect(sStyle.GetParentCache.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), 100, sStyle.GetActiveColor, 0, ssRectangle);
              // Draw shadow
              FadeRect(sStyle.FCacheBmp.Canvas, Rect(sStyle.Shadow.Offset, sStyle.Shadow.Offset, Width + sStyle.Shadow.Offset, Height + sStyle.Shadow.Offset), sStyle.FCacheBmp.Canvas, Point(sStyle.Shadow.Offset, sStyle.Shadow.Offset), sStyle.Shadow.Transparency, sStyle.Shadow.Color, sStyle.Shadow.Blur, ssRectangle);
              // Fading
              FadeRect(sStyle.FCacheBmp.Canvas, Rect(0, 0, Width, Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, sStyle.GetActiveColor, 0, ssRectangle);
            end
            else begin
              // Fading
              FadeRect(sStyle.GetParentCache.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, sStyle.GetActiveColor, 0, ssRectangle);
            end;
          end
          else Alert;
        end;
      end;
      bgsImage: begin
        if Assigned(sStyle.Background.Image) and Assigned(sStyle.Background.Image.Graphic) and (sStyle.Background.Image.Width <> 0) and (sStyle.Background.Image.Height <> 0) then begin
          TileBitmap(sStyle.FCacheBmp.Canvas, aRect, sStyle.Background.Image.Graphic);
//            FadeRect(Self, sStyle.FCacheBmp.Canvas, Rect(0, 0, Width, Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, Color, 0, ssRectangle);
        end;
      end;
      bgsGradient: begin
        case sStyle.Background.Gradient.Layout of
          gtTopBottom, gtLeftRight : begin
            GradientBmp(sStyle.FCacheBmp, ClientRect, sStyle.Background.Gradient.Color1, sStyle.Background.Gradient.Color2, sStyle.Background.Gradient.Layout);
          end;
        end;
      end;
    end;

    inc(aRect.Left, Margin);
    dec(aRect.Right, Margin);
    WriteText(aRect, sStyle);
    DrawPanelBorders(sStyle.FCacheBmp.Canvas);
    sStyle.CopyFromCache(Canvas.Handle, 0, 0, Width, Height);
{
    for i := 0 to ControlCount - 1 do begin
      if GetStyleInfo(Controls[i]) > 1 then begin
        case GetsStyle(Controls[i]).COC of
          COC_TsEdit..COC_TsSpinEdit : begin
            TsEdit(Controls[i]).Repaint;
          end;
          COC_TsCustomComboBox..COC_TsBDEComboBox : begin
          end;
          COC_TsDateEdit : begin
          end;
          COC_TsCheckedControl..COC_TsRadioButton : begin
            TsCheckedControl(Controls[i]).PaintControl
          end;
          COC_TsCurrencyEdit : begin
            TsCurrencyEdit(Controls[i]).Repaint;
          end;
          COC_TsPanel..COC_TsCustomPanel: begin
            TsCustomPanel(Controls[i]).Repaint
          end;
          COC_TsPageControl: begin
            TsPageControl(Controls[i]).Repaint // DrawTabs(Rect(0, 0, TsPageControl(Controls[i]).Width, TsPageControl(Controls[i]).Height))
          end;
          COC_TsCustomButton..COC_TsColorSelect : begin
            TsCustomButton(Controls[i]).Repaint
          end;
        end;
      end;
    end;
}
//  PaintControls(Canvas.Handle, Self);
//  end;

end;

procedure TsCustomPanel.DrawPanelBorders(Canvas: TCanvas);
var
  R: TRect;
begin
  GetWindowRect(Handle, R);
  OffsetRect(R, -R.Left, -R.Top);
  if BevelOuter in [bvRaised] then begin
    PaintPanelBorder(Canvas.Handle, R.Left, R.Top, R.Right, R.Bottom, False, sStyle);
  end
  else if BevelOuter in [bvSpace, bvLowered] then begin
    PaintPanelBorder(Canvas.Handle, R.Left, R.Top, R.Right, R.Bottom, True, sStyle);
  end;
  InflateRect(R, - BorderWidth - BevelWidth, - BorderWidth - BevelWidth);

  if BevelInner in [bvRaised] then begin
    PaintPanelBorder(Canvas.Handle, R.Left, R.Top, R.Right, R.Bottom, False, sStyle);
  end
  else if BevelInner in [bvSpace, bvLowered] then begin
    PaintPanelBorder(Canvas.Handle, R.Left, R.Top, R.Right, R.Bottom, True, sStyle);
  end;
end;

procedure TsCustomPanel.WndProc(var Message: TMessage);
begin
  if Assigned(FsStyle) then FsStyle.WndProc(Message);
  if Message.Result <> 1 then
    inherited;
end;

{ TsPanel }

constructor TsPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  sStyle.COC := COC_TsPanel;
end;

{ TsContainer }

constructor TsContainer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  sStyle.COC := COC_TsContainer;
  BorderStyle := bsSingle;
  BevelOuter := bvNone;
  sStyle.Shadow.DontUse := True;
end;

{ TsGlassPanel }

constructor TsGlassPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  sStyle.COC := COC_TsGlassPanel;
  FMask := TPicture.Create;
end;

procedure TsGlassPanel.SetMask(const Value: TPicture);
begin
  if Assigned(Value) then begin
    FMask.Assign(Value);
    FMask.Bitmap.PixelFormat := pf24bit;
    FMask.Bitmap.Monochrome := True;
  end
  else begin
    FMask.Assign(nil);
  end;
  Invalidate;
end;

procedure TsGlassPanel.Paint;
var
  R: TRect;
  i: Integer;
begin
  if not sStyle.FDrawingStop then begin
    Color := sStyle.GetActiveColor;
    if sStyle.Flat then begin
      R := ClientRect;
      sStyle.FCacheBmp.Height := R.Bottom - R.Top;
      sStyle.FCacheBmp.Width := R.Right - R.Left;

      if sStyle.Painting.Transparency = 0 then begin
        sStyle.FCacheBmp.Canvas.Brush.Color := Color;
        sStyle.FCacheBmp.Canvas.FillRect(R);
      end else if Assigned(sStyle.Background.Image) and Assigned(sStyle.Background.Image.Graphic) and (sStyle.Background.Image.Width <> 0) and (sStyle.Background.Image.Height <> 0) then begin
        TileBitmap(sStyle.FCacheBmp.Canvas, R, sStyle.Background.Image.Graphic);
        if Assigned(Mask) then begin
          FadeByMask(sStyle.FCacheBmp, Mask.Graphic, Rect(0, 0, Width, Height), sStyle.Painting.Transparency, Color);
        end
        else begin
          FadeRect(sStyle.FCacheBmp.Canvas, Rect(0, 0, Width, Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, Color, 0, ssRectangle);
        end;
      end
      else begin
        if sStyle.GetParentCache <> nil then begin
          if sStyle.Shadow.Enabled then begin
            // Copying area
            FadeRect(sStyle.GetParentCache.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), 100, Color, 0, ssRectangle);
            // Draw shadow
            FadeRect(sStyle.FCacheBmp.Canvas, Rect(sStyle.Shadow.Offset, sStyle.Shadow.Offset, Width + sStyle.Shadow.Offset, Height + sStyle.Shadow.Offset), sStyle.FCacheBmp.Canvas, Point(sStyle.Shadow.Offset, sStyle.Shadow.Offset), sStyle.Shadow.Transparency, sStyle.Shadow.Color, sStyle.Shadow.Blur, ssRectangle);
            // Fading
            FadeRect(sStyle.FCacheBmp.Canvas, Rect(0, 0, Width, Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, Color, 0, ssRectangle);
          end
          else begin
            if Assigned(Mask) and not Mask.Bitmap.Empty then begin
              FadeRect(sStyle.GetParentCache.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), 100, Color, 0, ssRectangle);
              FadeByMask(sStyle.FCacheBmp, Mask.Graphic, Rect(0, 0, Width, Height), sStyle.Painting.Transparency, Color);
            end
            else begin
              FadeRect(sStyle.GetParentCache.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), sStyle.Painting.Transparency, sStyle.GetActiveColor, 0, ssRectangle);
            end;
          end;
        end
        else begin
          if True then begin
            sStyle.FCacheBmp.Canvas.Brush.Color := Color;
            sStyle.FCacheBmp.Canvas.FillRect(R);
          end;
        end;

      end;
      WriteText(R, sStyle);
      DrawGlassBorders;
      Canvas.CopyRect(ClientRect, sStyle.FCacheBmp.Canvas, ClientRect);
    end
    else inherited;
    for i := 0 to ControlCount - 1 do begin
      if Controls[i] is TsPageControl then
        TsPageControl(Controls[i]).Repaint
      else if Controls[i] is TsCheckedControl then
        TsCheckedControl(Controls[i]).DrawCheckText;
    end;
  end;
end;

procedure TsGlassPanel.DrawGlassBorders;
begin
  //
end;

{ TsToolBar }

procedure TsToolBar.AdjustSize;
var
  aRect, NewRect : TRect;
  ParBorder : integer;
  function GetMaxRect : TRect; var i: integer; begin
    Result.Left := 2048;
    Result.Top := 1536;
    Result.Right := 0;
    Result.Bottom := 0;
    for i := 0 to ControlCount - 1 do begin
      if Controls[i] is TControl then begin
        Result.Left   := Mini(TControl(Controls[i]).Left, Result.Left);
        Result.Right  := Maxi(TControl(Controls[i]).Left + TControl(Controls[i]).Width, Result.Right);
        Result.Top    := Mini(TControl(Controls[i]).Top, Result.Top);
        Result.Bottom := Maxi(TControl(Controls[i]).Top + TControl(Controls[i]).Height, Result.Bottom);
      end;
    end;
    aRect.Right := Maxi(aRect.Right, aRect.Left + 12);
    aRect.Bottom := Maxi(aRect.Bottom, aRect.Top + 12);
    if BevelOuter <> bvNone then begin
      InflateRect(Result, BevelWidth, BevelWidth);
    end;
    if BevelInner <> bvNone then begin
      InflateRect(Result, BevelWidth, BevelWidth);
    end;
    InflateRect(Result, BorderWidth, BorderWidth);
  end;
begin
{    SetWindowPos(Handle, 0, 0, 0, Width, Height, SWP_NOACTIVATE or SWP_NOMOVE or
      SWP_NOZORDER);
    RequestAlign;}

  //inherited;
  NewRect := BoundsRect;
  if FAutoSize and (ControlCount > 0) and not DontAutoSize and not (csLoading in ComponentState) then begin
    aRect := GetMaxRect;
    if Parent is TsCustomPanel
      then ParBorder := 0//TsCustomPanel(Parent).BorderWidth
      else ParBorder := 0;
//    if Parent is TWinControl then ParBorder := TWinControl(Parent).BorderWidth;
    case Align of
      alNone   : begin
        NewRect.Right := NewRect.Left + (aRect.Right - aRect.Left);
        NewRect.Bottom := NewRect.Top + (aRect.Bottom - aRect.Top);
      end;
      alLeft   : begin
        NewRect.Right := aRect.Right - aRect.Left + ParBorder;
      end;
      alTop    : begin
        NewRect.Bottom := NewRect.Top + aRect.Bottom - aRect.Top + ParBorder;
      end;
      alRight  : begin
        NewRect.Left := NewRect.Right - (aRect.Right - aRect.Left) - ParBorder;
      end;
      alBottom : begin
        NewRect.Top := NewRect.Bottom - (aRect.Bottom - aRect.Top) - ParBorder;
      end;
    end;
  end;
  SetBounds(NewRect.Left, NewRect.Top, NewRect.Right - NewRect.Left, NewRect.Bottom - NewRect.Top);
  RequestAlign;
end;

constructor TsToolBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  sStyle.COC := COC_TsToolBar;
  Caption := ' ';
  DontAutoSize := False;
  FButtonWidth := 22;
  FButtonHeight := 22;
end;

procedure TsToolBar.SetAutoSize(const Value: boolean);
begin
  if FAutoSize <> Value then begin
    FAutoSize := Value;
    Invalidate;
  end;
end;

procedure TsToolBar.SetImages(Index: integer; Value: TCustomImageList);
var
  i : integer;
begin
  case Index of
    0: begin
      FImages := Value;
      for i := 0 to ControlCount - 1 do begin
        if Controls[i] is TsSpeedButton then begin
          TsSpeedButton(Controls[i]).Images := Value;
//          TsSpeedButton(Controls[i])
        end;
      end;
    end;
    1: begin
      FImagesHot := Value;
      for i :=0 to ControlCount - 1 do begin
        if Controls[i] is TsSpeedButton then begin
          TsSpeedButton(Controls[i]).ImagesHot := Value;
        end;
      end;
    end;
    2: begin
      FImagesDisabled := Value;
      for i :=0 to ControlCount - 1 do begin
        if Controls[i] is TsSpeedButton then begin
          TsSpeedButton(Controls[i]).ImagesDisabled := Value;
        end;
      end;
    end;
  end;
end;

procedure TsToolBar.SetShowCaptions(const Value: boolean);
var
  i : integer;
begin
  if FShowCaptions <> Value then begin
    FShowCaptions := Value;
    for i :=0 to ControlCount - 1 do begin
      if Controls[i] is TsSpeedButton then begin
        TsSpeedButton(Controls[i]).ShowCaption := Value;
      end;
    end;
  end;
end;

{ TsToolBarEditor }

procedure TsToolBarEditor.ExecuteVerb(Index: Integer);
var
  NewButton : TsSpeedButton;
begin
  (Component as TsToolBar).DontAutoSize := True;
  NewButton := TsSpeedButton.Create(Designer.GetRoot);
  NewButton.Parent := (Component as TsToolBar);
  NewButton.Align := alLeft;
  NewButton.sStyle.Background.Mode := bgsTransparent;
  NewButton.sStyle.Shadow.DontUse := True;
  NewButton.sStyle.Painting.Transparency := 50;
  NewButton.sStyle.Painting.Bevel := bvNone;
  NewButton.sStyle.Selection.Border.Width := 1;
  NewButton.sStyle.Selection.Border.Bevel := bsFlat;

  NewButton.ShowCaption := False;

  NewButton.Left := (Component as TsToolBar).Width - 1;

  NewButton.Images := (Component as TsToolBar).Images;
  NewButton.ImagesHot := (Component as TsToolBar).ImagesHot;
  NewButton.ImagesDisabled := (Component as TsToolBar).ImagesDisabled;

  NewButton.sStyle.Painting.ColorActive := scLightViolet;

  NewButton.Height := (Component as TsToolBar).FButtonHeight;
  NewButton.Width  := (Component as TsToolBar).FButtonWidth;

  case Index of
    0:  begin
      NewButton.ShowCaption := (Component as TsToolBar).ShowCaptions;
      NewButton.Name := Designer.UniqueName('TsToolButton');
      NewButton.Caption := NewButton.Name;
    end;
    1:  begin
      NewButton.ButtonStyle := tbsDivider;
      NewButton.sStyle.Selection.Mode := [];
      NewButton.sStyle.Painting.Bevel := bvLowered;
      NewButton.Name := Designer.UniqueName('TsToolDivider');
    end;
    2:  begin
      NewButton.ButtonStyle := tbsSeparator;
      NewButton.Name := Designer.UniqueName('TsToolSeparator');
    end;
  end;
  NewButton.AutoSize := True;
  (Component as TsToolBar).DontAutoSize := True;
  Designer.SelectComponent(NewButton);
  if Designer <> nil then Designer.Modified;
end;

function TsToolBarEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0:  result := 'New button';
    1:  result := 'New divider';
    2:  result := 'New separator';
  end;
end;

function TsToolBarEditor.GetVerbCount: Integer;
begin
  result := 3;
end;


end.
